﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Messaging;
using System.Configuration;

namespace MSMQManager
{
    public class MSMQHelper
    {
        private static readonly string queuePath = ConfigurationManager.AppSettings["EmailQueuePath"];
        private static int queueTimeout = 50;
        protected MessageQueueTransactionType transactionType = MessageQueueTransactionType.Automatic;
        protected MessageQueue queue;
        protected TimeSpan timeout;

        public MSMQHelper()
        {
            //if (!MessageQueue.Exists(queuePath))
            //    queue = MessageQueue.Create(queuePath);
            //else
                queue = new MessageQueue(queuePath);
            this.timeout = TimeSpan.FromSeconds(Convert.ToDouble(queueTimeout));

            //// Performance optimization since we don't need these features
            queue.DefaultPropertiesToSend.AttachSenderId = false;
            queue.DefaultPropertiesToSend.UseAuthentication = false;
            queue.DefaultPropertiesToSend.UseEncryption = false;
            queue.DefaultPropertiesToSend.AcknowledgeType = AcknowledgeTypes.None;
            queue.DefaultPropertiesToSend.UseJournalQueue = false;
            queue.Formatter = new BinaryMessageFormatter();

        }

        public void Send(MessageBase msg)
        {
            transactionType = MessageQueueTransactionType.None;
            Message message = new Message();
            message.Body = msg;
            message.Formatter = new BinaryMessageFormatter();
            queue.Send(message, msg.Id.ToString(), transactionType);
        }

        public void Receive()
        {
            try
            {
                transactionType = MessageQueueTransactionType.None;

                using (Message message = queue.Receive(timeout, transactionType))
                {
                    var msg = (MessageBase)message.Body;
                    Console.WriteLine(msg.ToString());
                }

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message.ToString());
            }
        }
    }
}
